module TestCatElements
using Test
using Catlab, Catlab.Theories, Catlab.Graphs, Catlab.CategoricalAlgebra

arr = @acset Graph begin
  V = 2
  E = 1
  src=[1]
  tgt=[2]
end
elarr = elements(arr)
@testset "Arrow Elements" begin
  @test nparts(elarr, :El)  == 3
  @test nparts(elarr, :Arr) == 2
  @test nparts(elarr, :Ob)  == 2
  @test nparts(elarr, :Hom) == 2
  @test elarr[:, :πₑ]    == [1,1,2]
  @test elarr[:, :πₐ]    == [1,2]
  @test elarr[:, :src]   == [3,3]
  @test elarr[:, :tgt]   == [1,2]
  @test elarr[:, :nameo] == [:V, :E]
  @test elarr[:, :nameh] == [:src, :tgt]
  @test elarr[:, :dom]   == [2, 2]
  @test elarr[:, :cod]   == [1, 1]
end

b = @acset Graph begin
  V = 2
  E = 1
  src = [1]
  tgt = [2]
end

ThBPG, obmap, hommap = CatElements.presentation(elements(b))
@test Symbol.(generators(ThBPG)) == [:E_1, :V_1, :V_2, :tgt_E_1, :src_E_1]


p₀ = @acset Graph begin
  V = 2
  E = 2
  src=[1,2]
  tgt=[2,1]
end

ThPetri, obmap, hommap = CatElements.presentation(elements(p₀))
@test Symbol.(generators(ThPetri)) == [:E_1, :E_2, :V_1, :V_2, :src_E_2, :tgt_E_1, :tgt_E_2, :src_E_1]

@acset_type Petri(ThPetri)

sir_eltsch = @acset Petri begin
  V_1 = 3
  V_2 = 2
  E_1 = 3
  E_2 = 3

  src_E_1 = [1,2,2]
  tgt_E_1 = [1,1,2]

  src_E_2 = [1,1,2]
  tgt_E_2 = [2,2,3]
end

@testset "DiBipartite Elements" begin
  @test length(generators(ThPetri)) == 8
  @test nparts(sir_eltsch, :V_1) == 3 
  @test nparts(sir_eltsch, :V_2) == 2
  @test nparts(sir_eltsch, :E_1) == 3
  @test nparts(sir_eltsch, :E_2) == 3
  @test sir_eltsch[:, :src_E_2] == [1,1,2]
end

end
